package main.java.com.rotaryknobview;

import android.content.Context;
import android.widget.ImageView;
import android.view.View;
import android.view.ViewGroup;
import android.graphics.drawable.Drawable;
import android.view.MotionEvent;
import android.util.AttributeSet;
import android.graphics.Canvas;


import anywheresoftware.b4a.BA;

public class RotaryKnobView extends ImageView  {

  private float angle = 0f;
  private float theta_old=0f;
  
  private RotaryKnobListener listener;
  
  public interface RotaryKnobListener {
    public void onKnobChanged(int arg, float angle);
  }
  
  public void setKnobListener(RotaryKnobListener l )
  {
    listener = l;
  }
  
  public RotaryKnobView(Context context) {
    super(context);
    initialize();
  }
  
  public RotaryKnobView(Context context, AttributeSet attrs)
  {
    super(context, attrs);
    initialize();
  }
  
  public RotaryKnobView(Context context, AttributeSet attrs, int defStyle)
  {
    super(context, attrs, defStyle);
    initialize();
  }
  
  private float getTheta(float x, float y)
  {
    float sx = x - (getWidth() / 2.0f);
    float sy = y - (getHeight() / 2.0f);
    
    float length = (float)Math.sqrt( sx*sx + sy*sy);
    float nx = sx / length;
    float ny = sy / length;
    float theta = (float)Math.atan2( ny, nx );
    
    final float rad2deg = (float)(180.0/Math.PI);
    float thetaDeg = theta*rad2deg;
    
    return (thetaDeg < 0) ? thetaDeg + 360.0f : thetaDeg;
  }
  
  public void initialize()
  {
    this.setImageResource(R.drawable.jog);
    setOnTouchListener(new OnTouchListener()
      {
 @Override
 public boolean onTouch(View v, MotionEvent event) {
   float x = event.getX(0);
   float y = event.getY(0);
   float theta = getTheta(x,y);

   switch(event.getAction() & MotionEvent.ACTION_MASK)
     {
     case MotionEvent.ACTION_POINTER_DOWN:
       theta_old = theta;
       break;
     case MotionEvent.ACTION_MOVE:
       invalidate();
       float delta_theta = theta - theta_old;
       theta_old = theta;
       int direction = (delta_theta > 0) ? 1 : -1;
       angle += step*direction;
	   if (angle < 0.0f) angle = 0.0f;
	   
	   if (angle > numberofturns * 360) angle = numberofturns * 360;

       notifyListener(direction, angle);
       break;
     }
   return true;
 }
      });
  }
  
  private void notifyListener(int dir, float angl)
  {
    if (null!=listener)
      listener.onKnobChanged(dir, angl);
  }
  
  protected void onDraw(Canvas c)
  {
    c.rotate(angle,getWidth()/2,getHeight()/2);   
    super.onDraw(c);
  }
  
  private float step = 1.0f;
  public void setadjustmentStep(float step){ 
     this.step = step;
  }
  
  private int numberofturns = 1;
  public void setnumberOfTurns(int numberofturns) {
	  this.numberofturns = numberofturns;
  }
  
 
} 
 



/* 
public class RotaryKnobView extends ImageView  {

    private float angle = 0f;
    private float theta_old=0f;
    private int width;
    private int height;
    private int direction = 0;
	private float prevangle = 0f;
   
    private RotaryKnobListener listener;
    
    public interface RotaryKnobListener {
      public void onKnobChanged(int direction, float angle);
    }
    
    public void setKnobListener(RotaryKnobListener l )
    {
      listener = l;
    }
    
    public RotaryKnobView(Context context) {
      super(context);
    // TODO Auto-generated constructor stub
    }
    
    public RotaryKnobView(Context context, AttributeSet attrs)
    {
      super(context, attrs);
      initialize();
    }
    
    public RotaryKnobView(Context context, AttributeSet attrs, int defStyle)
    {
      super(context, attrs, defStyle);
    }
    
    protected void onSizeChanged (int w, int h, int oldw, int oldh) {
      width = w;
    	height = h;
        initialize();
    }
    
    private float getTheta(float x, float y)
    {
      float sx = x - (width / 2.0f);
      float sy = y - (height / 2.0f);
 
      float length = (float)Math.sqrt( sx*sx + sy*sy);
      float nx = sx / length;
      float ny = sy / length;
      float theta = (float)Math.atan2( ny, nx );
 
      final float rad2deg = (float)(180.0/Math.PI);
      float theta2 = theta*rad2deg;
 
      return (theta2 < 0) ? theta2 + 360.0f : theta2;
    }
    
    public void initialize()
    {
 
      this.setImageResource(R.drawable.jog);
      setOnTouchListener(new OnTouchListener()
      {
        @Override
        public boolean onTouch(View v, MotionEvent event) {
          // TODO Auto-generated method stub
          int action = event.getAction();
          int actionCode = action & MotionEvent.ACTION_MASK;
          if (actionCode == MotionEvent.ACTION_DOWN)             //MotionEvent.ACTION_POINTER_DOWN
          {
            float x = event.getX(0);
            float y = event.getY(0);
            theta_old = getTheta(x, y);
          }
          else if (actionCode == MotionEvent.ACTION_MOVE)
          {
            invalidate();
       
            float x = event.getX(0);
            float y = event.getY(0);
       
            float theta = getTheta(x, y);
            float delta_theta = theta - theta_old;
       
            theta_old = theta;
       
            int direction = (delta_theta > 0) ? 1 : -1;
            angle = theta - 270;

			angle = (theta + 90) % 360;
			BA.Log("ANGLE = " + angle);
			

			if ((angle > prevangle) && (angle < 360.0f) && (direction == -1)) {
				direction = 1;
				prevangle = angle;
			} else {
				prevangle = angle;
			}	
			
			if ((angle < prevangle) && (angle > 0.0f) && (direction == 1)) {
				direction = -1;
				prevangle = angle;
			} else {
				prevangle = angle;
			}				
			
			

       
            notifyListener(direction, angle);
 
          }
          return true;
        }

     });
    }
    
    private void notifyListener(int direction, float angle) {
    	if (null!=listener) {
    		listener.onKnobChanged(direction, angle);
    	}
    }
    
    protected void onDraw(Canvas c) {
    	c.rotate(angle,width/2,height/2);   
    	super.onDraw(c);
    }
    
}
  */







